{---------------------------------------------------------------------------------------

    The purpose of this component is to allow applications to easily
    incorporate a phone number field that will convert alpha characters into
    their numeric equivelent on the phone dial-pad.

    This component is used just like any edit component, with the addition of
    mapping output to other edit controls.

    To parse the names from the field, the control must first have a reference to
    five other standard TEdit controls.  These must be referenced according to the
    type of data you wish for them to contain.  The fields to reference are as follows:

    NumericField

    AlphaNumericField

    LocaleField

    CountryField

    If the StateLookup or CountryLookup does not work on every case, you may add cases
    to the StringList directly after inserting the component, or you may dynamically
    load the data from a flat text file (Formatted like a TStringList) at runtime
    for extra functionality.

---------------------------------------------------------------------------------------}
                              {Copyright  1996,1997 by Jim Wood}

unit DTMFEdit;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Menus,
  Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, DsgnIntf;

type
  TDTMFEdit = class(TEdit)
  private
    FAbout: TStringList;
    FAutoFormat : boolean;
    FNumericConversion : boolean;
    FAllowQandZ : boolean;
    FNumericField : TEdit;
    FAlphaNumericField : TEdit;
    FCountryField : TEdit;
    FLocaleField : TEdit;
    FInternationalLookup : TStringList;
    FDomesticLookup : TStringList;
    function  GetEditText: string;
    function  GetAlphaNumeric(ParseText: string): string;
    function  GetNumeric(ParseText: string): string;
    function  GetDomesticCountry(ParseText: string): string;
    function  GetDomesticLocale(ParseText: string): string;
    function  GetInternationalCountry(ParseText: string): string;
    function  GetInternationalLocale(ParseText: string): string;
    procedure SetEditText(const Value: string);
    procedure SetInternational(Value:TStringList);
    procedure SetDomestic(Value:TStringList);
    procedure SetAbout(Value:TStringList);
  protected
    function  StripLeadingSpaces(var StripString: string): string;
    function  StripExtraSpaces(var StripString: string): string;
    procedure KeyUp(var Key: Word; Shift: TShiftState); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress( var Key: char ); override;
    procedure Change; override;
    procedure GetSel(var SelStart: Integer; var SelStop: Integer);
    procedure SetSel(SelStart: Integer; SelStop: Integer);
    procedure CheckCursor;
  public
    constructor Create(AOwner:TComponent);override;
    destructor Destroy;override;
    function  GetTextLen: Integer;
    procedure ParseDTMF;
  published
    property About: TStringList read FAbout write SetAbout;
    property AutoFormat : boolean read FAutoFormat write FAutoFormat;
    property NumericConversion : boolean read FNumericConversion write FNumericConversion;
    property AllowQandZ : boolean read FAllowQandZ write FAllowQandZ;
    property NumericField : TEdit read FNumericField write FNumericField;
    property AlphaNumericField : TEdit read FAlphaNumericField write FAlphaNumericField;
    property CountryField : TEdit read FCountryField write FCountryField;
    property LocaleField : TEdit read FLocaleField write FLocaleField;
    property InternationalLookup : TStringList read FInternationalLookup write SetInternational;
    property DomesticLookup : TStringList read FDomesticLookup write SetDomestic;
    property AutoSelect;
    property AutoSize;
    property BorderStyle;
    property CharCase;
    property Color;
    property Ctl3D;
    property Cursor;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property Height;
    property HelpContext;
    property HideSelection;
    property Hint;
    property Left;
    property MaxLength;
    property Name;
    property OEMConvert;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Tag;
    property Text;
    property Top;
    property Visible;
    property Width;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
end;

var
EditField,
SetUpCaseSelection,
DontShowChar,
BackspacePressed,
ReFormatField: boolean;
OldSelStart,
MoveStringBack,
CharDTMFPos,
CharDTMFLength,
CharDTMFStop,
CharDTMFPos2,
CharDTMFLength2,
CharDTMFStop2,
CheckErr: integer;
DTMF,
TempDTMF: array [0..255] of Char;

procedure Register;

implementation

{================================================================================}

procedure Register;
begin
  RegisterComponents('Edit Controls',[TDTMFEdit]);
end;
 
{================================================================================}

procedure TDTMFEdit.KeyPress( var Key: Char );
var
 CheckDTMF: string;
begin
 EditField := true;
 GetSel(CharDTMFPos, CharDTMFStop);
 CharDTMFLength := CharDTMFStop-CharDTMFPos;
 DontShowChar := false;
 ReformatField := false;
 if Key = ^M then Exit;
 FillChar(DTMF, SizeOf(DTMF), #0);
 StrPCopy(DTMF, GetEditText);
 if ( (BackSpacePressed = false) and (Key < '0') or (Key > '9')) then
  begin
   DontShowChar := true;
  end;
 if FAutoFormat = false then Exit;
 CheckDTMF := StrPas(DTMF);
 if ((StrLen(DTMF) = 7) and (DTMF[7] = '-')) then
  begin
   StrLCopy(TempDTMF, DTMF, 6);
   StrCopy(DTMF, TempDTMF);
   SetEditText(StrPas(DTMF));
  end;
 if ((StrLen(DTMF) = 3) and (DTMF[3] = '-') and (BackSpacePressed = true)) then
  begin
   StrLCopy(TempDTMF, DTMF, 3);
   StrCopy(DTMF, TempDTMF);
   SetEditText(StrPas(DTMF));
  end;
 if ((StrLen(DTMF) = 4) and (DTMF[3] <> '-') and (DTMF[0] <> '(')) then
  begin
   StrCat(DTMF, ' ');
   DTMF[4] := DTMF[3];
   DTMF[3] := '-';
   SetEditText(StrPas(DTMF));
   SetSel(StrLen(DTMF),StrLen(DTMF));
  end;
 if ((StrLen(DTMF) = 14) and (DTMF[1] = '1')) then
  begin
   MoveStringBack := 0;
   while MoveStringBack <= 14 do
    begin
     DTMF[MoveStringBack] := DTMF[MoveStringBack+1];
     Inc(MoveStringBack);
    end;
   DTMF[0] := '(';
   DTMF[3] := DTMF[5];
   DTMF[4] := ')';
   DTMF[5] := ' ';
   DTMF[8] := DTMF[9];
   DTMF[9] := '-';
   SetEditText(StrPas(DTMF));
   SetSel(StrLen(DTMF),StrLen(DTMF));
   {CheckAreaCode;}
  end;
if (StrLen(DTMF) = 14) and (DTMF[0] = '(') and
  (DTMF[4] = ')') and (DTMF[9] = '-') and (BackspacePressed = true) then
  begin
   StrCopy(TempDTMF, '   -      ');
   TempDTMF[0] := DTMF[1];
   TempDTMF[1] := DTMF[2];
   TempDTMF[2] := DTMF[3];
   TempDTMF[4] := DTMF[6];
   TempDTMF[5] := DTMF[7];
   TempDTMF[6] := DTMF[8];
   TempDTMF[7] := DTMF[10];
   TempDTMF[8] := DTMF[11];
   TempDTMF[9] := DTMF[12];
   SetEditText(StrPas(TempDTMF));
   SetSel(StrLen(TempDTMF),StrLen(TempDTMF));
   StrPCopy(DTMF, GetEditText);
   BackSpacePressed := false;
   DontShowChar := false;
   ReformatField := true;
  end;
 if (StrLen(DTMF) = 14) and (DTMF[1] <> '0') and
    (CharDTMFPos = StrLen(DTMF)) then
  begin
    MessageBeep(0);
    StrLCopy(TempDTMF, DTMF, 13);
    StrCopy(DTMF, TempDTMF);
    SetEditText(StrPas(DTMF));
    SetSel(StrLen(DTMF),StrLen(DTMF));
  end;
 if (StrLen(DTMF) = 10) and (DTMF[0] <> '(') and (Key <> Char($08)) then
  begin
   if (DTMF[0] = '0') then
   begin
    StrCopy(TempDTMF, '(   )-       ');
    TempDTMF[1] := DTMF[0];
    TempDTMF[2] := DTMF[1];
    TempDTMF[3] := DTMF[2];
    TempDTMF[6] := DTMF[4];
    TempDTMF[7] := DTMF[5];
    TempDTMF[8] := DTMF[6];
    TempDTMF[9] := DTMF[7];
    TempDTMF[10] := DTMF[8];
    TempDTMF[11] := DTMF[9];
    TempDTMF[12] := DTMF[10];
    SetEditText(StrPas(TempDTMF));
    SetSel(StrLen(TempDTMF),StrLen(TempDTMF));
    StrPCopy(DTMF, GetEditText);
    {EditSTATE.Field.AsString := ''; }
    {CheckCountryCode;}
   end else
   begin
    StrCopy(TempDTMF, '(   )    -    ');
    TempDTMF[1] := DTMF[0];
    TempDTMF[2] := DTMF[1];
    TempDTMF[3] := DTMF[2];
    TempDTMF[6] := DTMF[4];
    TempDTMF[7] := DTMF[5];
    TempDTMF[8] := DTMF[6];
    TempDTMF[10] := DTMF[7];
    TempDTMF[11] := DTMF[8];
    TempDTMF[12] := DTMF[9];
    TempDTMF[13] := DTMF[10];
    SetEditText(StrPas(TempDTMF));
    SetSel(StrLen(TempDTMF),StrLen(TempDTMF));
    StrPCopy(DTMF, GetEditText);
    {CheckAreaCode;}
   end;
  end;
 BackSpacePressed := false;
 inherited KeyPress(Key);
end;

{================================================================================}

procedure TDTMFEdit.GetSel(var SelStart: Integer; var SelStop: Integer);
var
  Res: LongInt;
begin
  Res := SendMessage(Handle, EM_GETSEL, 0, 0);
  SelStart := LoWord(Res);
  SelStop  := HiWord(Res);
end;

{================================================================================}

procedure TDTMFEdit.SetSel(SelStart: Integer; SelStop: Integer);
{var
  Res: LongInt;}
begin
  // Res := MakeLong(SelStart, SelStop);
  SendMessage(Handle, EM_SETSEL, SelStart, SelStop);
end;

{================================================================================}

procedure TDTMFEdit.CheckCursor;
begin
  if not HandleAllocated then  Exit;
end;

{================================================================================}

procedure TDTMFEdit.SetEditText(const Value: string);
var
  Buffer: array[0..255] of Char;
begin
  if GetEditText <> Value then
  begin
    SetTextBuf(StrPCopy(Buffer, Value));
    CheckCursor;
  end;
end;

{================================================================================}

function TDTMFEdit.GetEditText: string;
begin
  Result := inherited Text;
end;

{================================================================================}

function TDTMFEdit.GetTextLen: Integer;
begin
  Result := Length(Text);
end;

{================================================================================}

procedure TDTMFEdit.Change;
var
    GetText,
    PutText: String;
begin
 if FNumericConversion = true then
  begin
  try
  GetSel(CharDTMFPos2, CharDTMFStop2);
  except
    Exit;
  end; 
  GetText := GetEditText;
  CharDTMFLength2 := CharDTMFStop2-CharDTMFPos;
   if (CharDTMFPos2 = 0) and (CharDTMFLength2 = 0) then Exit;
   if DontShowChar = false then
    begin
     inherited Change;
     Exit;
    end;
    Fillchar(DTMF, Sizeof(DTMF), #0);
    StrPCopy(DTMF, GetText);
    if DTMF[0] = #0
      then
       begin
        inherited Change;
        Exit;
       end;
   StrLCopy(TempDTMF, DTMF, StrLen(DTMF)-1);
   case Upcase(DTMF[StrLen(DTMF)-1]) of
    'A'..'C'   : StrCat(TempDTMF, '2');
    'D'..'F'   : StrCat(TempDTMF, '3');
    'G'..'I'   : StrCat(TempDTMF, '4');
    'J'..'L'   : StrCat(TempDTMF, '5');
    'M'..'O'   : StrCat(TempDTMF, '6');
    'P'        : StrCat(TempDTMF, '7');
    'Q'        : begin
                   if FAllowQandZ = FALSE then
                    begin
                     DTMF[StrLen(DTMF)-1] := #0;
                     MessageBeep(0);
                    end
                   else
                  StrCat(TempDTMF, '7');
                 end;
    'R'..'S'   : StrCat(TempDTMF, '7');
    'T'..'V'   : StrCat(TempDTMF, '8');
    'W'..'Y'   : StrCat(TempDTMF, '9');
    'Z'        : begin
                   if FAllowQandZ = FALSE then
                    begin
                     DTMF[StrLen(DTMF)-1] := #0;
                     MessageBeep(0);
                    end
                   else
                  StrCat(TempDTMF, '9');
                 end;
    '!'..'/'   : begin MessageBeep(0); StrCopy(DTMF, TempDTMF); end;
    '['..'~'   : begin MessageBeep(0); StrCopy(DTMF, TempDTMF); end;
    '@'        : begin MessageBeep(0); StrCopy(DTMF, TempDTMF); end;
    ':'        : begin MessageBeep(0); StrCopy(DTMF, TempDTMF); end;
    ';'        : begin MessageBeep(0); StrCopy(DTMF, TempDTMF); end;
    '='        : begin MessageBeep(0); StrCopy(DTMF, TempDTMF); end;
    '<'        : begin MessageBeep(0); StrCopy(DTMF, TempDTMF); end;
    '>'        : begin MessageBeep(0); StrCopy(DTMF, TempDTMF); end;
    '?'        : begin MessageBeep(0); StrCopy(DTMF, TempDTMF); end;
    end;
   if StrLen(TempDTMF) < StrLen(DTMF) then
    begin
     StrCat(TempDTMF, ' ');
     TempDTMF[StrLen(TempDTMF)-1] := DTMF[StrLen(DTMF)-1];
    end;
   PutText := StrPas(TempDTMF);
   SetEditText(StripLeadingSpaces(PutText));
   SetSel(StrLen(TempDTMF),StrLen(TempDTMF));;
 end
 else
  begin
  try
  GetSel(CharDTMFPos2, CharDTMFStop2);
  except
   begin
    inherited Change;
    Exit;
   end;
  end; 
  GetText := GetEditText;
  CharDTMFLength2 := CharDTMFStop2-CharDTMFPos;
   if (CharDTMFPos2 = 0) and (CharDTMFLength2 = 0) then
    begin
     inherited Change;
     Exit;
    end;
   if DontShowChar = false then
    begin
     inherited Change;
     Exit;
    end;
    Fillchar(DTMF, Sizeof(DTMF), #0);
    StrPCopy(DTMF, GetText);
    if DTMF[0] = #0
      then
       begin
        inherited Change;
        Exit;
       end;
   StrLCopy(TempDTMF, DTMF, StrLen(DTMF)-1);
   case Upcase(DTMF[StrLen(DTMF)-1]) of
    'Q'        : begin
                   if FAllowQandZ = FALSE then
                    begin
                     DTMF[StrLen(DTMF)-1] := #0;
                     MessageBeep(0);
                    end
                   else
                    begin
                     if DTMF[StrLen(DTMF)-1] = 'q' then
                      StrCat(TempDTMF, 'q')
                     else
                      StrCat(TempDTMF, 'Q')
                    end;
                 end;
    'Z'        : begin
                   if FAllowQandZ = FALSE then
                    begin
                     DTMF[StrLen(DTMF)-1] := #0;
                     MessageBeep(0);
                    end
                   else
                    begin
                     if DTMF[StrLen(DTMF)-1] = 'z' then
                      StrCat(TempDTMF, 'z')
                     else
                      StrCat(TempDTMF, 'Z')
                    end;
                 end;
    '!'..'/'   : begin MessageBeep(0); StrCopy(DTMF, TempDTMF); end;
    '['..'~'   : begin MessageBeep(0); StrCopy(DTMF, TempDTMF); end;
    '@'        : begin MessageBeep(0); StrCopy(DTMF, TempDTMF); end;
    ':'        : begin MessageBeep(0); StrCopy(DTMF, TempDTMF); end;
    ';'        : begin MessageBeep(0); StrCopy(DTMF, TempDTMF); end;
    '='        : begin MessageBeep(0); StrCopy(DTMF, TempDTMF); end;
    '<'        : begin MessageBeep(0); StrCopy(DTMF, TempDTMF); end;
    '>'        : begin MessageBeep(0); StrCopy(DTMF, TempDTMF); end;
    '?'        : begin MessageBeep(0); StrCopy(DTMF, TempDTMF); end;
    end;
   if StrLen(TempDTMF) < StrLen(DTMF) then
    begin
     StrCat(TempDTMF, ' ');
     TempDTMF[StrLen(TempDTMF)-1] := DTMF[StrLen(DTMF)-1];
    end;
   PutText := StrPas(TempDTMF);
   SetEditText(StripLeadingSpaces(PutText));
   SetSel(StrLen(TempDTMF),StrLen(TempDTMF));;
 end;
 inherited Change;
end;

{================================================================================}

procedure TDTMFEdit.KeyUp(var Key: Word; Shift: TShiftState);
begin
 if ReformatField = true then
  begin
   StrPCopy(DTMF, GetEditText);
   StrCat(DTMF, ' ');
   SetEditText(StrPas(TempDTMF));
   SetSel(StrLen(TempDTMF),StrLen(TempDTMF));
  end;
 if DontShowChar = false then
  begin
   inherited KeyUp(Key, Shift);
   Exit;
  end;
 if Key = VK_RETURN then Exit;
 inherited KeyUp(Key, Shift);
end;

{================================================================================}

procedure TDTMFEdit.KeyDown(var Key: Word; Shift: TShiftState);
var
 DTMF3: array [0..40] of Char;
begin
 if Key = VK_BACK then
  begin
   BackspacePressed := true;
  end;
 if Key = VK_UP then
  begin
  end;
 if (Key = VK_RETURN) or (Key = VK_DOWN) then
  begin
   StrPCopy(DTMF3, GetEditText);
   if StrComp(DTMF3, '') = 0 then
    begin
     {Make Noise?}
     {MessageBeep(0);}
    end;
   {Send Message to user if return is pressed}
  end;
 inherited KeyDown(Key, Shift);
end;

{================================================================================}

procedure TDTMFEdit.ParseDTMF;
var
  SplitString,
  TempAreaCode,
  CountryCode,
  GetText: String;
  FindToken: integer;
begin
 {Insert Parsing Routine here.}
   if Assigned(NumericField) then FNumericField.Text := '';
   if Assigned(AlphaNumericField) then FAlphaNumericField.Text := '';
   if Assigned(LocaleField) then FLocaleField.Text := '';
   if Assigned(CountryField) then FCountryField.Text := '';
   GetText := GetEditText;
   if Length(GetText) = 0 then
    begin
     Exit;
    end;
   if Pos('3035371079', GetText) > 0 then
    begin
     MessageBox(Handle, 'Thank you for using the TDTMFEdit component Written by: Jim Wood', 'TDTMFEdit Component', mb_OK);
    end;

   if Assigned(AlphaNumericField) then FAlphaNumericField.Text := GetAlphaNumeric(GetText);
   if Assigned(NumericField) then FNumericField.Text := GetNumeric(GetText);

   {Check for International LongDistance Dialing prefix}
   if Pos('(011)', GetText) > 0 then

   {Begin International Parser}
   begin
    for FindToken := 0 to FInternationalLookup.Count-1 do
     begin
      CountryCode := '-'+Copy(FInternationalLookup[FindToken],1,Pos(';',FInternationalLookup[FindToken])-1);
      if Pos(CountryCode, GetText) > 0 then
       begin
        if Assigned(CountryField) then FCountryField.Text := GetInternationalCountry(InternationalLookup[FindToken]);
        if Assigned(LocaleField) then FLocaleField.Text := GetInternationalLocale(InternationalLookup[FindToken]);
       end;
     end;
 end
   {End International Parser}

   else

   {Begin Domestic Parser}
   begin
    for FindToken := 0 to FDomesticLookup.Count-1 do
     begin
      SplitString := Copy(GetText,2,Pos(';',FDomesticLookup[FindToken])-1);
      TempAreaCode := Copy(FDomesticLookup[FindToken],1,Pos(';',FDomesticLookup[FindToken])-1);
      if SplitString = TempAreaCode then
       begin
        if Assigned(CountryField) then FCountryField.Text := GetDomesticCountry(FDomesticLookup[FindToken]);
        if Assigned(LocaleField) then FLocaleField.Text := GetDomesticLocale(FDomesticLookup[FindToken]);
       end;
     end;
   end;
   {End Domestic Parser}
end;

{================================================================================}

function TDTMFEdit.StripExtraSpaces(var StripString: string): string;
var
 StripPos,
 StripExtra: integer;
 TempStrip1,
 TempStrip2: string;
begin
 if FNumericConversion = true then
 begin
  for StripPos := 1 to Length(StripString) do
   begin
    StripExtra := Pos('  ', StripString);
    if StripExtra > 0 then
     begin
      TempStrip1 := Copy(StripString, 0, StripExtra-1);
      TempStrip2 := Copy(StripString, StripExtra+1, Length(StripString)-StripExtra+1);
      StripString := TempStrip1 + TempStrip2;
     end;
   end;
 end;
 Result := StripString;
end;

{================================================================================}

function TDTMFEdit.GetAlphaNumeric(ParseText: string): string;
var
 ConvertText: integer;
 AlphaNumericText: string;
begin
 AlphaNumericText := '';
 for ConvertText := 1 to Length(ParseText) do
  begin
   case Upcase(ParseText[ConvertText]) of
    'A'..'Z'   : AlphaNumericText := AlphaNumericText + ParseText[ConvertText];
    '0'..'9'   : AlphaNumericText := AlphaNumericText + ParseText[ConvertText];
   end;
  end;
 Result := AlphaNumericText;
end;

{================================================================================}

function TDTMFEdit.GetNumeric(ParseText: string): string;
var
 ConvertAlpha: integer;
 NumericText: string;
begin
 NumericText := '';
 for ConvertAlpha := 1 to Length(ParseText) do
  begin
   case Upcase(ParseText[ConvertAlpha]) of
    'A'..'Z'   : NumericText := NumericText + ParseText[ConvertAlpha];
    '0'..'9'   : NumericText := NumericText + ParseText[ConvertAlpha];
   end;
  end;
 for ConvertAlpha := 1 to Length(NumericText) do
   case Upcase(NumericText[ConvertAlpha]) of
    'A'..'C'   : NumericText[ConvertAlpha] := '2';
    'D'..'F'   : NumericText[ConvertAlpha] := '3';
    'G'..'I'   : NumericText[ConvertAlpha] := '4';
    'J'..'L'   : NumericText[ConvertAlpha] := '5';
    'M'..'O'   : NumericText[ConvertAlpha] := '6';
    'P'..'S'   : NumericText[ConvertAlpha] := '7';
    'T'..'V'   : NumericText[ConvertAlpha] := '8';
    'W'..'Z'   : NumericText[ConvertAlpha] := '9';
   end;
 Result := NumericText;
end;

{================================================================================}

function TDTMFEdit.GetDomesticCountry(ParseText: string): string;
var
 CountryText: string;
begin
 CountryText := ParseText;
 CountryText[Pos(';', CountryText)] := '~';
 if Pos(';', CountryText) > 0 then
  Result := Copy(CountryText, Pos('~',CountryText)+1, Pos(';',CountryText)-(Pos('~',CountryText)+1))
 else Result := '';
end;

{================================================================================}

function TDTMFEdit.GetDomesticLocale(ParseText: string): string;
var
 LocaleText: string;
begin
 LocaleText := ParseText;
 LocaleText[Pos(';', LocaleText)] := '~';
 if Pos(';',LocaleText) > 0 then
  Result := Copy(LocaleText, Pos(';',LocaleText)+1, Length(LocaleText))
 else Result := '';
end;

{================================================================================}

function TDTMFEdit.GetInternationalCountry(ParseText: string): string;
var
 CountryText: string;
begin
 CountryText := ParseText;
 if Pos(';', CountryText) > 0 then
  begin
   CountryText[Pos(';', CountryText)] := '~';
   Result := Copy(CountryText, Pos('~',CountryText)+1, Pos(';',CountryText)-(Pos('~',CountryText)+1));
  end
 else Result := '';
end;

{================================================================================}

function TDTMFEdit.GetInternationalLocale(ParseText: string): string;
var
 LocaleText: string;
begin
 LocaleText := ParseText;
 LocaleText[Pos(';', LocaleText)] := '~';
 if Pos(';',LocaleText) > 0 then
  Result := Copy(LocaleText, Pos(';',LocaleText)+1, Length(LocaleText))
 else Result := '';
end;

{================================================================================}

function TDTMFEdit.StripLeadingSpaces(var StripString: string): string;
begin
 if FNumericConversion = true then
 begin
 Result := Trim(StripString);
 end
else Result := StripString;
end;

{================================================================================}

constructor TDTMFEdit.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
     CharCase := ecUpperCase;
     MaxLength := 40;
     FAutoFormat := TRUE;
     FNumericConversion := TRUE;
     FAllowQandZ := FALSE;
     FInternationalLookup := TStringList.Create;
     FDomesticLookup := TStringList.Create;
     FAbout := TStringList.Create;

     {Create default greeting list}
     FAbout.Clear;
     FAbout.Add('TDTMFEdit - Version 1.1');
     FAbout.Add('---------------------------------');
     FAbout.Add(' ');
     FAbout.Add('This component is distributed as freeware.');
     FAbout.Add(' ');
     FAbout.Add('E-Mail comments or questions to: jwood@itp-cti.com');

     {Create default International Lookup list}
     FInternationalLookup.Clear;
     FInternationalLookup.Add('355;Albania;');
     FInternationalLookup.Add('35552;Albania;Durres');
     FInternationalLookup.Add('35542;Albania;Tirana');
     FInternationalLookup.Add('213;Algeria;');
     FInternationalLookup.Add('2137;Algeria;Adrar');
     FInternationalLookup.Add('2135;Algeria;Bajaia');
     FInternationalLookup.Add('684;Amer. Samoa;');
     FInternationalLookup.Add('54;Argentina;');
     FInternationalLookup.Add('541;Argentina;Buenos Aires');
     FInternationalLookup.Add('5451;Argentina;Cordoba');
     FInternationalLookup.Add('374;Armenia;');
     FInternationalLookup.Add('297;Aruba;');
     FInternationalLookup.Add('2978;Aruba;all points');
     FInternationalLookup.Add('61;Australia;');
     FInternationalLookup.Add('616;Australia;Canberra');
     FInternationalLookup.Add('613;Australia;Melbourne');
     FInternationalLookup.Add('612;Australia;Sydney');
     FInternationalLookup.Add('43;Austria;');
     FInternationalLookup.Add('43316;Austria;Graz');
     FInternationalLookup.Add('43512;Austria;Innsbruck');
     FInternationalLookup.Add('431;Austria;Vienna');
     FInternationalLookup.Add('973;Bahrain;');
     FInternationalLookup.Add('880;Bangladesh;');
     FInternationalLookup.Add('8802;Bangladesh;Dhaka');
     FInternationalLookup.Add('32;Belgium;');
     FInternationalLookup.Add('323;Belgium;Antwerp');
     FInternationalLookup.Add('322;Belgium;Brussels');
     FInternationalLookup.Add('501;Belize;');
     FInternationalLookup.Add('5012;Belize;Belize City');
     FInternationalLookup.Add('591;Boliva;');
     FInternationalLookup.Add('5912;Boliva;La Paz');
     FInternationalLookup.Add('387;Bosnia and Herzegovina;');
     FInternationalLookup.Add('38771;Bosnia;Sarajevo');
     FInternationalLookup.Add('55;Brazil;');
     FInternationalLookup.Add('5561;Brazil;Brasilia');
     FInternationalLookup.Add('5521;Brazil;Rio de Janeiro');
     FInternationalLookup.Add('5511;Brazil;Sao Paulo');
     FInternationalLookup.Add('359;Bulgaria;');
     FInternationalLookup.Add('3592;Bulgaria;Sofia');
     FInternationalLookup.Add('56;Chile;');
     FInternationalLookup.Add('562;Chile;Santiago');
     //Left off here
     FInternationalLookup.Add('86;China;');
     FInternationalLookup.Add('57;Columbia;');
     FInternationalLookup.Add('506;Costa Rica;');
     FInternationalLookup.Add('357;Cyprus;');
     FInternationalLookup.Add('42;Czechoslovakia;');
     FInternationalLookup.Add('45;Denmark;');
     FInternationalLookup.Add('593;Equador;');
     FInternationalLookup.Add('20;Egypt;');
     FInternationalLookup.Add('503;El Salvador;');
     FInternationalLookup.Add('372;Estonia;');
     FInternationalLookup.Add('251;Ethiopia;');
     FInternationalLookup.Add('679;Fiji Islands;');
     FInternationalLookup.Add('358;Finland;');
     FInternationalLookup.Add('33;France;');
     FInternationalLookup.Add('689;French Polynesia;');
     FInternationalLookup.Add('49;Germany;');
     FInternationalLookup.Add('30;Greece;');
     FInternationalLookup.Add('671;Guam;');
     FInternationalLookup.Add('502;Guatemala;');
     FInternationalLookup.Add('592;Guyana;');
     FInternationalLookup.Add('509;Haiti;');
     FInternationalLookup.Add('504;Honduras;');
     FInternationalLookup.Add('852;Hong Kong;');
     FInternationalLookup.Add('36;Hungary;');
     FInternationalLookup.Add('354;Iceland;');
     FInternationalLookup.Add('91;India;');
     FInternationalLookup.Add('62;Indonesia;');
     FInternationalLookup.Add('98;Iran;');
     FInternationalLookup.Add('964;Iraq;');
     FInternationalLookup.Add('353;Ireland;');
     FInternationalLookup.Add('972;Israel;');
     FInternationalLookup.Add('39;Italy;');
     FInternationalLookup.Add('225;Ivory Coast;');
     FInternationalLookup.Add('81;Japan;');
     FInternationalLookup.Add('962;Jordan;');
     FInternationalLookup.Add('254;Kenya;');
     FInternationalLookup.Add('82;Korea;');
     FInternationalLookup.Add('965;Kuwait;');
     FInternationalLookup.Add('371;Latvia;');
     FInternationalLookup.Add('231;Liberia;');
     FInternationalLookup.Add('218;Libya;');
     FInternationalLookup.Add('41;Liechtenstein;');
     FInternationalLookup.Add('370;Lithuania;');
     FInternationalLookup.Add('352;Luxembourg;');
     FInternationalLookup.Add('60;Malaysia;');
     FInternationalLookup.Add('960;Maldives;');
     FInternationalLookup.Add('356;Malta;');
     FInternationalLookup.Add('230;Mauritius;');
     FInternationalLookup.Add('52;Mexico;');
     FInternationalLookup.Add('3393;Monaco;');
     FInternationalLookup.Add('212;Morocco;');
     FInternationalLookup.Add('31;Netherlands;');
     FInternationalLookup.Add('599;Netherlands Antilles;');
     FInternationalLookup.Add('64;New Zealand;');
     FInternationalLookup.Add('505;Nicaragua;');
     FInternationalLookup.Add('234;Nigeria;');
     FInternationalLookup.Add('47;Norway;');
     FInternationalLookup.Add('92;Pakistan;');
     FInternationalLookup.Add('507;Panama;');
     FInternationalLookup.Add('595;Paraguay;');
     FInternationalLookup.Add('51;Peru;');
     FInternationalLookup.Add('63;Philippnes;');
     FInternationalLookup.Add('48;Poland;');
     FInternationalLookup.Add('351;Portugal;');
     FInternationalLookup.Add('974;Qatar;');
     FInternationalLookup.Add('40;Romania;');
     FInternationalLookup.Add('7;Russia/Ukraine;');
     FInternationalLookup.Add('7095;Russia;Moscow');
     FInternationalLookup.Add('966;Saudi Arabia;');
     FInternationalLookup.Add('221;Senegal;');
     FInternationalLookup.Add('65;Singapore;');
     FInternationalLookup.Add('27;South Africa;');
     FInternationalLookup.Add('34;Spain;');
     FInternationalLookup.Add('94;Sri Lanka;');
     FInternationalLookup.Add('597;Suriname;');
     FInternationalLookup.Add('46;Sweden;');
     FInternationalLookup.Add('41;Switzerland;');
     FInternationalLookup.Add('886;Taiwan;');
     FInternationalLookup.Add('255;Tanzania;');
     FInternationalLookup.Add('66;Thailand;');
     FInternationalLookup.Add('216;Tunisia;');
     FInternationalLookup.Add('90;Turkey;');
     FInternationalLookup.Add('256;Uganda;');
     FInternationalLookup.Add('971;United Arab Emirates;');
     FInternationalLookup.Add('44;United Kingdom;');
     FInternationalLookup.Add('598;Uruguay;');
     FInternationalLookup.Add('39;Vatican City;');
     FInternationalLookup.Add('58;Venuzuela;');
     FInternationalLookup.Add('967;Yemen;');
     FInternationalLookup.Add('38;Yugoslavia;');
     FInternationalLookup.Add('243;Zaire;');
     FInternationalLookup.Add('260;Zambia;');
     FInternationalLookup.Add('263;Zimbabwe;');

     {Create default Domestic Lookup list}
     FDomesticLookup.Clear;
     FDomesticLookup.Add('809;Carribean;');
     FDomesticLookup.Add('403;Canada;Alberta');
     FDomesticLookup.Add('604;Canada;British Columbia');
     FDomesticLookup.Add('204;Canada;Manitoba');
     FDomesticLookup.Add('506;Canada;New Brunswick');
     FDomesticLookup.Add('709;Canada;Newfoundland');
     FDomesticLookup.Add('902;Canada;Nova Scotia');
     FDomesticLookup.Add('905;Canada;Ontario');
     FDomesticLookup.Add('519;Canada;Ontario');
     FDomesticLookup.Add('705;Canada;Ontario');
     FDomesticLookup.Add('613;Canada;Ontario');
     FDomesticLookup.Add('807;Canada;Ontario');
     FDomesticLookup.Add('416;Canada;Ontario');
     FDomesticLookup.Add('514;Canada;Quebec');
     FDomesticLookup.Add('418;Canada;Quebec');
     FDomesticLookup.Add('306;Canada;Saskatchewan');
     FDomesticLookup.Add('205;U.S.A.;Alabama');
     FDomesticLookup.Add('334;U.S.A.;Alabama');
     FDomesticLookup.Add('907;U.S.A.;Alaska');
     FDomesticLookup.Add('602;U.S.A.;Arizona');
     FDomesticLookup.Add('520;U.S.A.;Arizona');
     FDomesticLookup.Add('501;U.S.A.;Arkansas');
     FDomesticLookup.Add('714;U.S.A.;California');
     FDomesticLookup.Add('805;U.S.A.;California');
     FDomesticLookup.Add('619;U.S.A.;California');
     FDomesticLookup.Add('707;U.S.A.;California');
     FDomesticLookup.Add('209;U.S.A.;California');
     FDomesticLookup.Add('310;U.S.A.;California');
     FDomesticLookup.Add('213;U.S.A.;California');
     FDomesticLookup.Add('408;U.S.A.;California');
     FDomesticLookup.Add('510;U.S.A.;California');
     FDomesticLookup.Add('818;U.S.A.;California');
     FDomesticLookup.Add('909;U.S.A.;California');
     FDomesticLookup.Add('916;U.S.A.;California');
     FDomesticLookup.Add('415;U.S.A.;California');
     FDomesticLookup.Add('719;U.S.A.;Colorado');
     FDomesticLookup.Add('303;U.S.A.;Colorado');
     FDomesticLookup.Add('970;U.S.A.;Colorado');
     FDomesticLookup.Add('203;U.S.A.;Connecticut');
     FDomesticLookup.Add('302;U.S.A.;Delaware');
     FDomesticLookup.Add('305;U.S.A.;Florida');
     FDomesticLookup.Add('813;U.S.A.;Florida');
     FDomesticLookup.Add('904;U.S.A.;Florida');
     FDomesticLookup.Add('407;U.S.A.;Florida');
     FDomesticLookup.Add('912;U.S.A.;Georgia');
     FDomesticLookup.Add('404;U.S.A.;Georgia');
     FDomesticLookup.Add('706;U.S.A.;Georgia');
     FDomesticLookup.Add('808;U.S.A.;Hawaii');
     FDomesticLookup.Add('208;U.S.A.;Idaho');
     FDomesticLookup.Add('618;U.S.A.;Illinois');
     FDomesticLookup.Add('217;U.S.A.;Illinois');
     FDomesticLookup.Add('312;U.S.A.;Illinois');
     FDomesticLookup.Add('708;U.S.A.;Illinois');
     FDomesticLookup.Add('815;U.S.A.;Illinois');
     FDomesticLookup.Add('309;U.S.A.;Illinois');
     FDomesticLookup.Add('812;U.S.A.;Indiana');
     FDomesticLookup.Add('219;U.S.A.;Indiana');
     FDomesticLookup.Add('317;U.S.A.;Indiana');
     FDomesticLookup.Add('712;U.S.A.;Iowa');
     FDomesticLookup.Add('319;U.S.A.;Iowa');
     FDomesticLookup.Add('515;U.S.A.;Iowa');
     FDomesticLookup.Add('316;U.S.A.;Kansas');
     FDomesticLookup.Add('913;U.S.A.;Kansas');
     FDomesticLookup.Add('606;U.S.A.;Kentucky');
     FDomesticLookup.Add('502;U.S.A.;Kentucky');
     FDomesticLookup.Add('504;U.S.A.;Louisiana');
     FDomesticLookup.Add('318;U.S.A.;Louisiana');
     FDomesticLookup.Add('207;U.S.A.;Maine');
     FDomesticLookup.Add('410;U.S.A.;Maryland');
     FDomesticLookup.Add('301;U.S.A.;Maryland');
     FDomesticLookup.Add('617;U.S.A.;Massachusetts');
     FDomesticLookup.Add('508;U.S.A.;Massachusetts');
     FDomesticLookup.Add('413;U.S.A.;Massachusetts');
     FDomesticLookup.Add('313;U.S.A.;Michigan');
     FDomesticLookup.Add('517;U.S.A.;Michigan');
     FDomesticLookup.Add('616;U.S.A.;Michigan');
     FDomesticLookup.Add('810;U.S.A.;Michigan');
     FDomesticLookup.Add('906;U.S.A.;Michigan');
     FDomesticLookup.Add('218;U.S.A.;Minnesota');
     FDomesticLookup.Add('612;U.S.A.;Minnesota');
     FDomesticLookup.Add('507;U.S.A.;Minnesota');
     FDomesticLookup.Add('601;U.S.A.;Mississippi');
     FDomesticLookup.Add('314;U.S.A.;Missouri');
     FDomesticLookup.Add('417;U.S.A.;Missouri');
     FDomesticLookup.Add('816;U.S.A.;Missouri');
     FDomesticLookup.Add('406;U.S.A.;Montana');
     FDomesticLookup.Add('402;U.S.A.;Nebraska');
     FDomesticLookup.Add('308;U.S.A.;Nebraska');
     FDomesticLookup.Add('702;U.S.A.;Nevada');
     FDomesticLookup.Add('603;U.S.A.;New Hampshire');
     FDomesticLookup.Add('609;U.S.A.;New Jersey');
     FDomesticLookup.Add('201;U.S.A.;New Jersey');
     FDomesticLookup.Add('908;U.S.A.;New Jersey');
     FDomesticLookup.Add('505;U.S.A.;New Mexico');
     FDomesticLookup.Add('518;U.S.A.;New York');
     FDomesticLookup.Add('607;U.S.A.;New York');
     FDomesticLookup.Add('718;U.S.A.;New York');
     FDomesticLookup.Add('917;U.S.A.;New York');
     FDomesticLookup.Add('716;U.S.A.;New York');
     FDomesticLookup.Add('516;U.S.A.;New York');
     FDomesticLookup.Add('212;U.S.A.;New York');
     FDomesticLookup.Add('914;U.S.A.;New York');
     FDomesticLookup.Add('315;U.S.A.;New York');
     FDomesticLookup.Add('704;U.S.A.;North Carolina');
     FDomesticLookup.Add('919;U.S.A.;North Carolina');
     FDomesticLookup.Add('910;U.S.A.;North Carolina');
     FDomesticLookup.Add('701;U.S.A.;North Dakota');
     FDomesticLookup.Add('216;U.S.A.;Ohio');
     FDomesticLookup.Add('513;U.S.A.;Ohio');
     FDomesticLookup.Add('614;U.S.A.;Ohio');
     FDomesticLookup.Add('419;U.S.A.;Ohio');
     FDomesticLookup.Add('405;U.S.A.;Oklahoma');
     FDomesticLookup.Add('918;U.S.A.;Oklahoma');
     FDomesticLookup.Add('503;U.S.A.;Oregon');
     FDomesticLookup.Add('610;U.S.A.;Pennsylvania');
     FDomesticLookup.Add('814;U.S.A.;Pennsylvania');
     FDomesticLookup.Add('717;U.S.A.;Pennsylvania');
     FDomesticLookup.Add('215;U.S.A.;Pennsylvania');
     FDomesticLookup.Add('412;U.S.A.;Pennsylvania');
     FDomesticLookup.Add('401;U.S.A.;Rhode Island');
     FDomesticLookup.Add('803;U.S.A.;South Carolina');
     FDomesticLookup.Add('605;U.S.A.;South Dakota');
     FDomesticLookup.Add('615;U.S.A.;Tennessee');
     FDomesticLookup.Add('901;U.S.A.;Tennessee');
     FDomesticLookup.Add('915;U.S.A.;Texas');
     FDomesticLookup.Add('806;U.S.A.;Texas');
     FDomesticLookup.Add('512;U.S.A.;Texas');
     FDomesticLookup.Add('409;U.S.A.;Texas');
     FDomesticLookup.Add('214;U.S.A.;Texas');
     FDomesticLookup.Add('817;U.S.A.;Texas');
     FDomesticLookup.Add('713;U.S.A.;Texas');
     FDomesticLookup.Add('210;U.S.A.;Texas');
     FDomesticLookup.Add('903;U.S.A.;Texas');
     FDomesticLookup.Add('801;U.S.A.;Utah');
     FDomesticLookup.Add('802;U.S.A.;Vermont');
     FDomesticLookup.Add('703;U.S.A.;Virginia');
     FDomesticLookup.Add('804;U.S.A.;Virginia');
     FDomesticLookup.Add('360;U.S.A.;Washington');
     FDomesticLookup.Add('206;U.S.A.;Washington');
     FDomesticLookup.Add('509;U.S.A.;Washington');
     FDomesticLookup.Add('202;U.S.A.;Washington DC');
     FDomesticLookup.Add('304;U.S.A.;West Virginia');
     FDomesticLookup.Add('715;U.S.A.;Wisconsin');
     FDomesticLookup.Add('414;U.S.A.;Wisconsin');
     FDomesticLookup.Add('608;U.S.A.;Wisconsin');
     FDomesticLookup.Add('307;U.S.A.;Wyoming');
end;


{================================================================================}

procedure TDTMFEdit.SetAbout(Value:TStringList);
begin
  if FAbout <> Value then
     FAbout.Assign( Value );
     {Create default greeting list}
     MessageBeep(0);
     FAbout.Clear;
     FAbout.Add('TDTMFEdit - Version 1.1');
     FAbout.Add('---------------------------------');
     FAbout.Add(' ');
     FAbout.Add('This component is distributed as freeware.');
     FAbout.Add(' ');
     FAbout.Add('E-Mail comments or questions to: jwood@itp-cti.com');
end;

{================================================================================}

procedure TDTMFEdit.SetInternational(Value:TStringList);
begin
  if FInternationalLookup <> Value then
     FInternationalLookup.Assign( Value )
end;

{================================================================================}

procedure TDTMFEdit.SetDomestic(Value:TStringList);
begin
  if FDomesticLookup <> Value then
     FDomesticLookup.Assign( Value )
end;

{================================================================================}

destructor TDTMFEdit.Destroy;
begin
  FInternationalLookup.Free;
  FDomesticLookup.Free;
  inherited Destroy;
end;
end.

